from dotenv import dotenv_values

from riseact.client import execute_partner_query
from riseact.topics.app.data import Application


def app_get_config() -> dict:
    return dotenv_values(".env")


def app_get_by_client_id(*, client_id: str) -> Application | None:
    result = execute_partner_query(
        """
        query AppByClientId($clientId: String!) {
            appByClientId(clientId: $clientId) {
                id
                name
                type
                organization
                appUrl
                installUrl
                authorName
                authorHomepageUrl
                redirectUris
                clientId
                clientSecret
            }
        }
    """,
        variable_values={"clientId": client_id},
    )

    found_app = result["appByClientId"]

    if found_app:
        return Application(**found_app)


def app_get(*, id: int) -> Application | None:
    result = execute_partner_query(
        """
        query AppById($id: Int!) {
            app(id: $id) {
                id
                name
                type
                organization
                appUrl
                installUrl
                authorName
                authorHomepageUrl
                redirectUris
                clientId
                clientSecret
            }
        }
    """,
        variable_values={"id": id},
    )

    found_app = result["app"]

    if found_app:
        return Application(**found_app)


def app_search() -> list[Application]:
    result = execute_partner_query(
        """
        query Apps {
            apps {
                edges {
                    node {
                        id
                        name
                        type
                        organization
                        appUrl
                        installUrl
                        authorName
                        authorHomepageUrl
                        redirectUris
                        clientId
                        clientSecret
                    }
                }
            }
        }
    """
    )

    return [Application(**app["node"]) for app in result["apps"]["edges"]]



import os

import inquirer
import typer

from riseact import SETTINGS, ui
from riseact.topics.info.selectors import organization_search
from riseact.topics.theme.services import (
    theme_create_development_theme,
    theme_push_theme,
    theme_watch_current_folder,
)
from riseact.topics.theme.validators import theme_validate_path

app = typer.Typer()


@app.command()
def serve():
    cwd = os.getcwd()

    partner_organizations = organization_search()
    organization_id = inquirer.list_input(
        "Which organization would you like to use to serve your theme?",
        choices=[(o.name, o.id) for o in partner_organizations],
    )

    organization = None
    for o in partner_organizations:
        if o.id == organization_id:
            organization = o

    if organization is None:
        ui.console.print("[red]Error retrieving organization")
        return

    with ui.console.status("[magenta]Validating path..."):
        validation = theme_validate_path(cwd)
        if validation.error:
            ui.console.print(f"[bold red]ERROR:[/bold red] {validation.message}")
            raise typer.Exit(1)

    theme, assets = theme_create_development_theme(cwd)

    ui.console.print()
    ui.success("Theme uploaded")

    table = ui.table()
    table.add_row("Name", theme.name)
    table.add_row("UUID", theme.uuid)
    table.add_row("Preview", f"{organization.sitefrontUrl}?__preview={theme.uuid}")
    table.add_row("Editor", f"{SETTINGS.admin_host}/themes/{theme.uuid}/editor")
    ui.console.print(table)
    ui.console.print()

    with ui.console.status("[magenta]Watching for file changes"):
        theme_watch_current_folder(cwd, theme, assets)


@app.command()
def push():
    cwd = os.getcwd()

    with ui.console.status("[magenta]Validating path...") as status:
        validation = theme_validate_path(cwd)
        if validation.error:
            ui.console.print(f"[bold red]ERROR:[/bold red] {validation.message}")
            raise typer.Exit(1)

        status.update(status="[magenta]Retrieving Organization info...")

        partner_organizations = organization_search()
        organization_id = inquirer.list_input(
            "Which organization would you like to use to serve your theme?",
            choices=[(o.name, o.id) for o in partner_organizations],
        )

        organization = None
        for o in partner_organizations:
            if o.id == organization_id:
                organization = o

        if organization is None:
            ui.console.print("[red]Error retrieving organization")
            return

    theme, assets = theme_push_theme(cwd)

    ui.console.print()
    ui.success("Theme pushed")

    table = ui.table()
    table.add_row("Name", theme.name)
    table.add_row("UUID", theme.uuid)
    table.add_row("Preview", f"{organization.sitefrontUrl}?__preview={theme.uuid}")
    table.add_row("Editor", f"{SETTINGS.admin_host}/themes/{theme.uuid}/editor")
    ui.console.print(table)
    ui.console.print()



from dataclasses import dataclass
from typing import Literal, TypedDict


class OAuthParams(TypedDict):
    response_type: Literal["code"]
    code_challenge_method: Literal["S256"]
    code_challenge: str
    client_id: str
    redirect_uri: str


@dataclass
class AccessToken:
    access_token: str
    refresh_token: str


class OAuthTokenPayload(TypedDict):
    client_id: str
    code_verifier: str
    code: str
    redirect_uri: str
    grant_type: Literal["authorization_code"]


class OAuthRefreshTokenPayload(TypedDict):
    client_id: str
    refresh_token: str
    redirect_uri: str
    grant_type: Literal["refresh_token"]



from riseact.client import execute_partner_query, execute_query
from riseact.topics.info.data import Organization, Partner, User


def partner_get() -> Partner:
    result = execute_partner_query(
        """
        query Partner {
            partner {
                id
                name
            }
        }
    """
    )

    return Partner(**result["partner"])


def organization_search() -> list[Organization]:
    result = execute_partner_query(
        """
        query Organizations {
            organizations {
                edges {
                    node {
                        id
                        name
                        domain
                        sitefrontUrl
                    }
                }
            }
        }
    """
    )

    return [Organization(**org["node"]) for org in result["organizations"]["edges"]]


def organization_get() -> Organization:
    result = execute_query(
        """
        query OrganizationName {
            organizations {
                id
                name
                domain
                sitefrontUrl
            }
        }
    """
    )

    return Organization(**result["organization"])


def user_get() -> User:
    result = execute_query(
        """
        query UserCurrent {
            user {
                name
                email
            }
        }
    """
    )

    return User(**result["user"])



from dataclasses import dataclass
from enum import Enum
from fnmatch import fnmatch
from typing import Callable, Optional

from watchdog.events import FileSystemEventHandler

from riseact.topics.theme.data import Asset, AssetMap, Theme


class AssetEvent(Enum):
    CREATED = 1
    DELETED = 2
    MOVED = 3
    MODIFIED = 3


@dataclass
class AssetChange:
    event: AssetEvent
    key: str
    path: str
    dest_key: Optional[str] = None
    dest_path: Optional[str] = None
    asset: Optional[Asset] = None


AssetChangeHandler = Callable[[AssetChange, Theme, AssetMap], None]


def is_ignored_path(path: str):
    content = None

    with open(".riseactignore", "r") as f:
        content = f.read()

    for ignore in content.split("\n"):
        if fnmatch(path, ignore):
            print(f"check {path} {ignore} True")
            return True

    return False


class AssetEventHandler(FileSystemEventHandler):
    """Logs all the events captured."""

    path: str
    handler: AssetChangeHandler
    theme: Theme
    assets: AssetMap

    def __init__(
        self,
        *,
        path: str,
        theme: Theme,
        assets: AssetMap,
        handler: AssetChangeHandler,
    ):
        super().__init__()
        self.path = path
        self.handler = handler
        self.theme = theme
        self.assets = assets

    def on_moved(self, event):
        super().on_moved(event)
        asset_key = event.src_path.replace(self.path + "/", "")
        if event.is_directory or is_ignored_path(asset_key):
            return

        asset_dest_key = event.dest_path.replace(self.path + "/", "")
        self.handler(
            AssetChange(
                event=AssetEvent.MOVED,
                key=asset_key,
                path=event.src_path,
                dest_key=asset_dest_key,
                dest_path=event.dest_path,
                asset=self.assets[asset_key],
            ),
            self.theme,
            self.assets,
        )

    def on_created(self, event):
        super().on_created(event)
        asset_key = event.src_path.replace(self.path + "/", "")
        if event.is_directory or is_ignored_path(asset_key):
            return

        self.handler(
            AssetChange(
                event=AssetEvent.CREATED,
                key=asset_key,
                path=event.src_path,
            ),
            self.theme,
            self.assets,
        )

    def on_deleted(self, event):
        super().on_deleted(event)
        asset_key = event.src_path.replace(self.path + "/", "")
        if event.is_directory or is_ignored_path(asset_key):
            return

        self.handler(
            AssetChange(
                event=AssetEvent.DELETED,
                asset=self.assets[asset_key],
                key=asset_key,
                path=event.src_path,
            ),
            self.theme,
            self.assets,
        )

    def on_modified(self, event):
        super().on_modified(event)
        asset_key = event.src_path.replace(self.path + "/", "")
        if event.is_directory or is_ignored_path(asset_key):
            return

        self.handler(
            AssetChange(
                event=AssetEvent.MODIFIED,
                asset=self.assets[asset_key],
                key=asset_key,
                path=event.src_path,
            ),
            self.theme,
            self.assets,
        )



import hashlib
import json
import os
import time
from pathlib import Path
from typing import Iterator, Tuple

import magic
from watchdog.observers import Observer

from riseact import ui
from riseact.topics.theme.consts import ASSETS_BLACKLIST
from riseact.topics.theme.data import AssetMap, LocalAsset, Theme, ThemeManifest
from riseact.topics.theme.observer import AssetChange, AssetEvent, AssetEventHandler
from riseact.topics.theme.remote import (
    asset_create,
    asset_delete,
    asset_update,
    theme_create,
    theme_delete,
)


def theme_parse_manifest(path: str) -> ThemeManifest:
    manifest_path = Path(path) / "manifest.json"
    with open(manifest_path, "r") as f:
        content = f.read()

    data = json.loads(content)

    theme = ThemeManifest(**data)

    return theme


def theme_push_theme(path: str) -> Tuple[Theme, AssetMap]:
    manifest = theme_parse_manifest(path)
    theme = theme_create(manifest=manifest, dev=False)
    assets = theme_create_assets(path, theme)

    return theme, assets


def theme_create_development_theme(path: str) -> Tuple[Theme, AssetMap]:
    manifest = theme_parse_manifest(path)
    theme = theme_create(manifest=manifest, dev=False)
    assets = theme_create_assets(path, theme)

    return theme, assets


def theme_delete_development_theme(theme: Theme) -> None:
    theme_delete(theme)


def theme_create_assets(path: str, theme: Theme) -> AssetMap:
    assets_map: AssetMap = {}
    asset_paths = list(theme_scan_assets(path))

    with ui.progress() as progress:
        task = progress.add_task("[green]Uploading", total=len(asset_paths))

        for asset_path in asset_paths:
            asset_key = asset_path.replace(path, "")[1:]
            local_asset = theme_load_local_asset(asset_key, asset_path)

            asset = asset_create(local_asset, theme)
            assets_map[asset.key] = asset

            progress.update(task, advance=1)
            ui.success(asset.key)

    return assets_map


def theme_load_local_asset(asset_key: str, asset_path: str) -> LocalAsset:
    with open(asset_path, "r") as f:
        content = f.read()

    local_asset = LocalAsset(
        key=asset_key,
        value=content,
        contentType=magic.from_file(asset_path, mime=True),
        checksum=hashlib.md5(content.encode("utf-8")).hexdigest(),
    )

    return local_asset


def theme_local_assets(path: str) -> list[LocalAsset]:
    local_assets = []
    for asset_path in theme_scan_assets(path):
        asset_key = asset_path.replace(path, "")[1:]
        local_assets.append(theme_load_local_asset(asset_key, asset_path))
    return local_assets


def theme_scan_assets(path: str) -> Iterator[str]:
    for entry in os.scandir(path):
        for bl in ASSETS_BLACKLIST:
            if path.startswith(bl):
                continue
        if os.path.basename(entry.path) in ASSETS_BLACKLIST:
            continue
        if entry.is_dir(follow_symlinks=False):
            yield from theme_scan_assets(entry.path)
        else:
            yield entry.path


def theme_watch_current_folder(
    path: str,
    theme: Theme,
    assets: AssetMap,
):
    event_handler = AssetEventHandler(
        path=path,
        theme=theme,
        assets=assets,
        handler=theme_handle_changes,
    )
    observer = Observer()
    observer.schedule(event_handler, path, recursive=True)
    observer.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        theme_delete_development_theme(theme)
        observer.stop()

    observer.join()


def theme_handle_changes(
    change: AssetChange,
    theme: Theme,
    assets: AssetMap,
):
    try:
        if change.event == AssetEvent.CREATED:
            local_asset = theme_load_local_asset(change.key, change.path)
            ui.log_asset_change(f"[green]CREATED[/green] {change.key}")
            asset = asset_create(local_asset, theme)
            assets[asset.key] = asset
        elif change.event == AssetEvent.MODIFIED and change.asset:
            local_asset = theme_load_local_asset(change.key, change.path)
            ui.log_asset_change(f"[green]CHANGED[/green] {change.key}")
            asset_update(change.asset, local_asset, theme)
        elif change.event == AssetEvent.MOVED and change.asset:
            local_asset = theme_load_local_asset(change.key, change.path)
            ui.log_asset_change(f"[orange]MOVED[/orange] {change.key}")
            asset_update(change.asset, local_asset, theme)
        elif change.event == AssetEvent.DELETED and change.asset:
            ui.log_asset_change(f"[red]DELETED[/red] {change.key}")
            asset_delete(change.asset)
            del assets[change.key]
    except Exception as e:
        ui.console.print(
            f"ERROR: Cannot handle file change on {change.path}: {e}",
            style="red",
        )



import typer

from riseact import ui
from riseact.topics.info.selectors import organization_get, partner_get, user_get

app = typer.Typer()


@app.command()
def partner():
    with ui.console.status("[magenta]Retrieving informations..."):
        partner = partner_get()

        table = ui.table()
        table.add_row("Partner", partner.name)
        ui.console.print(table)


@app.command()
def organization():
    with ui.console.status("[magenta]Retrieving informations..."):
        organization = organization_get()

        table = ui.table()
        table.add_row("Organization", organization.name)
        table.add_row("Sitefront URL", organization.sitefrontUrl)
        ui.console.print(table)


@app.command()
def user():
    with ui.console.status("[magenta]Retrieving informations..."):
        user = user_get()

        table = ui.table()
        table.add_row("Name", user.name)
        table.add_row("Email", user.email)
        ui.console.print(table)



import typer
from rich.prompt import Prompt

from riseact import SETTINGS, ui
from riseact.client import client_init
from riseact.topics.app.commands import app as app_commands
from riseact.topics.auth.commands import app as auth_commands
from riseact.topics.auth.services import auth_login, auth_refresh_access_token
from riseact.topics.info.commands import app as info_commands
from riseact.topics.ngrok.commands import app as ngrok_commands
from riseact.topics.settings.commands import app as settings_commands
from riseact.topics.settings.services import settings_store_credentials
from riseact.topics.theme.commands import app as theme_commands

app = typer.Typer(
    rich_markup_mode=None,
    rich_help_panel=None,
)

app.add_typer(auth_commands, name="auth")
app.add_typer(settings_commands, name="settings")
app.add_typer(theme_commands, name="theme")
app.add_typer(info_commands, name="info")
app.add_typer(app_commands, name="app")
app.add_typer(ngrok_commands, name="ngrok")


@app.callback()
def warmup():
    if not SETTINGS.access_token:
        ui.console.print("To run this command, log in to Riseact Partners.")
        Prompt.ask("👉 Press any key to open the login page on your browser")
        auth_login()

    if SETTINGS.refresh_token:
        token = auth_refresh_access_token(SETTINGS.refresh_token)
        if token:
            settings_store_credentials(
                access_token=token.access_token,
                refresh_token=token.refresh_token,
            )

    client_init()


def run():
    app()


if __name__ == "__main__":
    run()



import os
from pathlib import Path

from riseact import SETTINGS, conf
from riseact.topics.settings.data import Settings, StoredSettings


def settings_create_default(filename: Path) -> None:
    data = StoredSettings().json()
    config_dir = os.path.dirname(filename)
    Path(config_dir).mkdir(exist_ok=True, parents=True)
    with open(filename, "w") as f:
        f.write(data)


def settings_store(settings: Settings):
    data = StoredSettings(
        access_token=settings.access_token,
        refresh_token=settings.refresh_token,
        ngrok_token=settings.ngrok_token,
    )

    if not os.path.exists(conf.APP_DIR):
        os.mkdir(conf.APP_DIR)

    with open(conf.SETTINGS_PATH, "w+") as f:
        f.write(data.json())


def settings_store_ngrok_token(*, token: str) -> None:
    SETTINGS.ngrok_token = token
    settings_store(SETTINGS)


def settings_store_credentials(
    *,
    access_token: str,
    refresh_token: str,
) -> None:
    SETTINGS.access_token = access_token
    SETTINGS.refresh_token = refresh_token
    settings_store(SETTINGS)


def settings_delete_credentials() -> None:
    SETTINGS.access_token = None
    SETTINGS.refresh_token = None
    settings_store(SETTINGS)
